package com.diagbot.client.hystrix;

import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/25 10:59
 */
@Component
@Slf4j
public class CdssCoreHystrix implements CdssCoreClient {
    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @Override
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        log.error("【hystrix】调用{}异常", "staticKnowledgeIndex");
        return null;
    }

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @Override
    public RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "updateHasInfoStatus");
        return null;
    }

    /**
     * 术语批量校验
     *
     * @param conceptVO
     * @return
     */
    @Override
    public RespDTO<List<String>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO) {
        log.error("【hystrix】调用{}异常", "getConceptNames");
        return null;
    }

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "index");
        return null;
    }

    /**
     * 开单合理性推理
     *
     * @param indicationPushVO
     * @return
     */
    @Override
    public RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO) {
        log.error("【hystrix】调用{}异常", "indication");
        return null;
    }
}
