package com.diagbot.facade;

import com.diagbot.dto.BillMsg;
import com.diagbot.entity.MappingShortEntity;
import com.diagbot.entity.ResultBill;
import com.diagbot.entity.ResultCritical;
import com.diagbot.entity.node.BillItem;
import com.diagbot.entity.node.LisCritical;
import com.diagbot.entity.node.MedNameRegName;
import com.diagbot.entity.node.TransfusionRemind;
import com.diagbot.enums.GraphLabelEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/8 10:27
 */
@Component
public class RuleConvertFacade {
    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    @Autowired
    private MedicineRepository medicineRepository;
    @Autowired
    private DrugTypeFacade drugTypeFacade;

    /**
     * 开单规则转换
     * @param billItems
     * @param hospitalId
     * @return
     */
    public List<ResultBill> billRuleConvert(List<BillItem> billItems, Long hospitalId) {
        List<ResultBill> retList = Lists.newLinkedList();
        List<ResultBill> resultBillList = Lists.newLinkedList();
        List<ResultBill> retMedNameList = Lists.newArrayList();
        List<ResultBill> retBillList = Lists.newArrayList();
        List<ResultBill> retConflictList = Lists.newArrayList();

        if (ListUtil.isEmpty(billItems)) {
            return retList;
        }

        List<MedNameRegName> medNameRegNameList = medicineRepository.findCustomByNameAndRegName(StatusEnum.Enable.getKey());
        Map<String, List<MedNameRegName>> medNameRegNameEntityMap
                = EntityUtil.makeEntityListMap(medNameRegNameList, "medRegName");
        Map<String, List<String>> medNameRegNameMap = new HashMap<>();
        medNameRegNameEntityMap.entrySet().forEach(entry -> {
            medNameRegNameMap.put(entry.getKey(), entry.getValue().stream().map(i -> i.getMedName()).collect(Collectors.toList()));
        });

        //药物过敏原药品分类转药品
        Map<String, List<String>> medClassMap = drugTypeFacade.getDrugTypeMap();

        Map<String, Map<String, Map<String, List<Long>>>> lisConfigMap = lisConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> pacsConfigMap = pacsConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, Map<String, List<Long>>>> drugConfigMap = drugConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> diseaseConfigMap = diseaseConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> operationConfigMap = operationConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        for (BillItem billItem : billItems) {
            //基础属性复制
            ResultBill resultBill = new ResultBill();
            resultBill.setBillType(1);
            resultBill.setBillItemType(billItem.getConceptLabel());
            resultBill.setBillItemName(billItem.getConceptName());
            resultBill.setConflictItemType(billItem.getConditionLabel());
            resultBill.setConflictItemName(billItem.getConditionName());
            if (billItem.getConditionMaxValue() != null) {
                resultBill.setMaxValue(BigDecimal.valueOf(billItem.getConditionMaxValue()));
            }
            if (billItem.getConditionMinValue() != null) {
                resultBill.setMinValue(BigDecimal.valueOf(billItem.getConditionMinValue()));
            }
            resultBill.setConflictItemRange(billItem.getConditionRange());
            resultBill.setUnit(billItem.getConditionUnit());

            if (resultBill.getConflictItemType().equals(GraphLabelEnum.MedAllergen.getName())) {
                List<String> medNameList = medClassMap.get(billItem.getConditionName());
                if (ListUtil.isNotEmpty(medNameList)) {
                    for (String medName : medNameList) {
                        ResultBill resultBill1Ext = new ResultBill();
                        BeanUtil.copyProperties(resultBill, resultBill1Ext);
                        resultBill1Ext.setReferenceValue(resultBill1Ext.getConflictItemName());
                        resultBill1Ext.setConflictItemName(medName);
                        resultBillList.add(resultBill1Ext);
                    }
                } else {
                    resultBillList.add(resultBill);
                }
            } else {
                resultBillList.add(resultBill);
            }
        }

        //药品注册名转通用名
        for (ResultBill resultBill : resultBillList) {
            if (resultBill.getBillItemType().equals(GraphLabelEnum.MedRegName.getName())) {
                List<String> medNameList = medNameRegNameMap.get(resultBill.getBillItemName());
                if (ListUtil.isNotEmpty(medNameList)) {
                    for (String medName : medNameList) {
                        ResultBill resultBill1Ext = new ResultBill();
                        BeanUtil.copyProperties(resultBill, resultBill1Ext);
                        resultBill1Ext.setReferenceValue(resultBill1Ext.getBillItemName());
                        resultBill1Ext.setBillItemName(medName);
                        retMedNameList.add(resultBill1Ext);
                    }
                } else {
                    resultBill.setSuccess(0);
                    resultBill.setMessage("药品注册名称找不到对应药品通用名称；");
                    retMedNameList.add(resultBill);
                }
            } else {
                retMedNameList.add(resultBill);
            }
        }

        //数据映射-开单项转换
        for (ResultBill resultBill : retMedNameList) {
            List<MappingShortEntity> billMappingList = Lists.newArrayList();
            if (resultBill.getBillItemType().equals(GraphLabelEnum.LisSet.getName())) {
                Map<String, Map<String, List<Long>>> subMap = lisConfigMap.get(resultBill.getBillItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        if (subEntry.getValue() != null && subEntry.getValue().size() > 0) {
                            subEntry.getValue().entrySet().forEach(detailEntry -> {
                                billMappingList.add(setMappingShortEntity(resultBill.getBillItemName(), subEntry.getKey(), detailEntry.getKey()));
                            });
                        } else {
                            billMappingList.add(setMappingShortEntity(resultBill.getBillItemName(), subEntry.getKey(), ""));
                        }
                    });
                }
            } else if (resultBill.getBillItemType().equals(GraphLabelEnum.PacsName.getName())
                    || resultBill.getBillItemType().equals(GraphLabelEnum.PacsSubName.getName())) {
                Map<String, List<Long>> subMap = pacsConfigMap.get(resultBill.getBillItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        billMappingList.add(setMappingShortEntity(resultBill.getBillItemName(), subEntry.getKey(), ""));
                    });
                }
            } else if (resultBill.getBillItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
                Map<String, List<Long>> subMap = operationConfigMap.get(resultBill.getBillItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        billMappingList.add(setMappingShortEntity(resultBill.getBillItemName(), subEntry.getKey(), ""));
                    });
                }
            } else if (resultBill.getBillItemType().equals(GraphLabelEnum.MedRegName.getName())) {
                if (medNameRegNameMap.containsKey(resultBill.getBillItemName())) {
                    Map<String, Map<String, List<Long>>> subMap = drugConfigMap.get(resultBill.getBillItemName());
                    if (subMap != null) {
                        subMap.entrySet().forEach(subEntry -> {
                            billMappingList.add(setMappingShortEntity(resultBill.getBillItemName(), subEntry.getKey(), ""));
                        });
                    }
                }
            }

            if (ListUtil.isEmpty(billMappingList)) {
                resultBill.setSuccess(0);
                resultBill.setMessage(StringUtil.isNotBlank(resultBill.getMessage())
                        ? resultBill.getMessage()
                        : ""
                        + "开单项缺少医院端映射；");
                retBillList.add(resultBill);
            } else {
                billMappingList.forEach(billMappingItem -> {
                    ResultBill addResultBill = new ResultBill();
                    BeanUtil.copyProperties(resultBill, addResultBill);
                    addResultBill.setBillItemHisName(billMappingItem.getHisName());
                    addResultBill.setBillItemHisDetailName(billMappingItem.getHisDetailName());
                    retBillList.add(addResultBill);
                });
            }
        }

        //数据映射-禁忌项转换
        for (ResultBill resultBill : retBillList) {
            List<MappingShortEntity> conflictMappingList = Lists.newArrayList();
            if (resultBill.getConflictItemType().equals(GraphLabelEnum.Lis.getName())) {
                Map<String, Map<String, List<Long>>> subMap = lisConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        if (subEntry.getValue() != null && subEntry.getValue().size() > 0) {
                            subEntry.getValue().entrySet().forEach(detailEntry -> {
                                conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), detailEntry.getKey()));
                            });
                        } else {
                            conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), ""));
                        }
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.BillConflictItem.getName())) {
                Map<String, List<Long>> subMap = pacsConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), ""));
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Disease.getName())) {
                Map<String, List<Long>> subMap = diseaseConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), ""));
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.OralMedicine.getName())) {
                Map<String, Map<String, List<Long>>> subMap = drugConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null) {
                    subMap.entrySet().forEach(subEntry -> {
                        conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), ""));
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.MedAllergen.getName())) {
                List<String> medNameList = medClassMap.get(resultBill.getConflictItemName());
                if (ListUtil.isNotEmpty(medNameList)) {
                    medNameList.forEach(medName -> {
                        Map<String, Map<String, List<Long>>> subMap = drugConfigMap.get(medName);
                        if (subMap != null) {
                            subMap.entrySet().forEach(subEntry -> {
                                conflictMappingList.add(setMappingShortEntity(resultBill.getConflictItemName(), subEntry.getKey(), ""));
                            });
                        }
                    });
                }
            }


            if (ListUtil.isEmpty(conflictMappingList)) {
                if (resultBill.getConflictItemType().equals(GraphLabelEnum.MedAllergen.getName())
                        || resultBill.getConflictItemType().equals(GraphLabelEnum.OralMedicine.getName())
                        || resultBill.getConflictItemType().equals(GraphLabelEnum.Lis.getName())
                        || resultBill.getConflictItemType().equals(GraphLabelEnum.BillConflictItem.getName())
                        || resultBill.getConflictItemType().equals(GraphLabelEnum.Disease.getName())) {
                    resultBill.setSuccess(0);
                    resultBill.setMessage(StringUtil.isNotBlank(resultBill.getMessage())
                            ? resultBill.getMessage()
                            : "" +
                            "禁忌项缺少医院端映射；");
                }
                retConflictList.add(resultBill);
            } else {
                conflictMappingList.forEach(conflictItem -> {
                    ResultBill addResultBill = new ResultBill();
                    BeanUtil.copyProperties(resultBill, addResultBill);
                    addResultBill.setConflictItemHisName(conflictItem.getHisName());
                    addResultBill.setConflictItemHisDetailName(conflictItem.getHisDetailName());
                    retConflictList.add(addResultBill);
                });
            }
        }

        //模拟输入值
        retList = createInputValue(retConflictList);
        return retList;
    }


    public List<ResultBill> billRuleConvert_transfusion(List<TransfusionRemind> items,Long hospitalId) {
        List<ResultBill> retList = Lists.newLinkedList();
        List<ResultBill> resultBillList = Lists.newLinkedList();
        List<ResultBill> retBillList = Lists.newArrayList();
        List<ResultBill> retConflictList = Lists.newArrayList();

        if (ListUtil.isEmpty(items)) {
            return retList;
        }

        Map<String, Map<String, Map<String, List<Long>>>> lisConfigMap = lisConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> diseaseConfigMap = diseaseConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> operationConfigMap = operationConfigFacade.getUniqueConfigMap(hospitalId, null, null);
        Map<String, Map<String, List<Long>>> transfusionConfigMap = transfusionConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        for (TransfusionRemind item : items) {
            //基础属性复制
            ResultBill resultBill = new ResultBill();
            resultBill.setBillType(2);
            resultBill.setBillItemType(GraphLabelEnum.TransfusionRemind.getName());
            resultBill.setBillItemName(item.getTransfusion_type());
            resultBill.setConflictItemType(item.getNode_label());
            resultBill.setConflictItemName(item.getIndex_name());
            if (item.getMaxval() != null) {
                resultBill.setMaxValue(BigDecimal.valueOf(item.getMaxval()));
            }
            if (item.getMinval() != null) {
                resultBill.setMinValue(BigDecimal.valueOf(item.getMinval()));
            }
            resultBill.setConflictItemRange(item.getRange());
            resultBill.setUnit(item.getUnit());
            resultBillList.add(resultBill);
        }

        //开单项转医院名称
        for (ResultBill resultBill : resultBillList) {
            Map<String, List<Long>> hisMap = transfusionConfigMap.get(resultBill.getBillItemName());
            if (hisMap != null && hisMap.size() > 0) {
                hisMap.entrySet().forEach(result -> {
                    ResultBill addResultBill = new ResultBill();
                    BeanUtil.copyProperties(resultBill, addResultBill);
                    addResultBill.setBillItemHisName(result.getKey());
                    retBillList.add(addResultBill);
                });
            } else {
                resultBill.setSuccess(0);
                resultBill.setMessage("开单项缺少医院端映射；");
                retBillList.add(resultBill);
            }
        }

        //禁忌项转医院名称
        for (ResultBill resultBill : retBillList) {
            Map<String, List<String>> hisMap = new HashMap<>();
            if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
                Map<String, List<Long>> subMap = operationConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null && subMap.size() > 0) {
                    subMap.entrySet().forEach(item -> {
                        hisMap.put(item.getKey(), new ArrayList<>());
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoDiseaseName.getName())) {
                Map<String, List<Long>> subMap = diseaseConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null && subMap.size() > 0) {
                    subMap.entrySet().forEach(item -> {
                        hisMap.put(item.getKey(), new ArrayList<>());
                    });
                }
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.LisName.getName())) {
                Map<String, Map<String, List<Long>>> subMap = lisConfigMap.get(resultBill.getConflictItemName());
                if (subMap != null && subMap.size() > 0) {
                    subMap.entrySet().forEach(item -> {
                        hisMap.put(item.getKey(), new ArrayList<>(item.getValue().keySet()));
                    });
                }
            }

            if (hisMap != null && hisMap.size() > 0) {
                hisMap.entrySet().forEach(item -> {
                    ResultBill addResultBill = new ResultBill();
                    BeanUtil.copyProperties(resultBill, addResultBill);
                    addResultBill.setConflictItemHisName(item.getKey());
                    if (ListUtil.isNotEmpty(item.getValue())) {
                        item.getValue().forEach(detailName -> {
                            ResultBill addResultBillExt = new ResultBill();
                            BeanUtil.copyProperties(addResultBill, addResultBillExt);
                            addResultBillExt.setConflictItemHisDetailName(detailName);
                            retConflictList.add(addResultBillExt);
                        });
                    } else {
                        retConflictList.add(addResultBill);
                    }
                });
            } else {
                resultBill.setSuccess(0);
                resultBill.setMessage("禁忌项缺少医院端映射；");
                retConflictList.add(resultBill);
            }
        }

        //模拟输入值
        retList = createInputValue(retConflictList);

        return retList;
    }

    public List<ResultCritical> criticalRuleConvert_lis(List<LisCritical> items,Long hospitalId) {
        List<ResultCritical> retList = Lists.newLinkedList();
        List<ResultCritical> resultList = Lists.newLinkedList();
        List<ResultCritical> resultMappingList = Lists.newArrayList();

        if (ListUtil.isEmpty(items)) {
            return retList;
        }

        Map<String, Map<String, Map<String, List<Long>>>> lisConfigMap = lisConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        for (LisCritical item : items) {
            //基础属性复制
            ResultCritical result = new ResultCritical();
            result.setCriticalType(1);
            result.setCriticalItemType(GraphLabelEnum.LisCritical.getName());

            String uniqueName = item.getName().substring(0, item.getName().length() - 3);
            result.setCriticalItemName(uniqueName);
            result.setReferenceValue(item.getName());
            if (StringUtil.isNotBlank(item.getMaxval())) {
                result.setMaxValue(BigDecimal.valueOf(Double.valueOf(item.getMaxval())));
            }
            if (StringUtil.isNotBlank(item.getMinval())) {
                result.setMinValue(BigDecimal.valueOf(Double.valueOf(item.getMinval())));
            }
            if (StringUtil.isNotBlank(item.getAge())) {
                String[] ages = item.getAge().split(":");
                if (ages.length == 1) {
                    if (item.getAge().startsWith(":")) {
                        result.setMaxAge(Integer.valueOf(ages[0]));
                    } else {
                        result.setMinAge(Integer.valueOf(ages[0]));
                    }
                } else if (ages.length > 1) {
                    result.setMinAge(Integer.valueOf(ages[0]));
                    result.setMaxAge(Integer.valueOf(ages[1]));
                }
            }
            result.setCriticalItemRange(StringUtil.isBlank(item.getRange()) ? 0 : Integer.valueOf(item.getRange()));
            result.setUnit(item.getUnit());
            resultList.add(result);
        }

        //医院端映射
        for (ResultCritical result : resultList) {
            Map<String, Map<String, List<Long>>> hisMap = lisConfigMap.get(result.getCriticalItemName());
            if (hisMap != null && hisMap.size() > 0) {
                hisMap.entrySet().forEach(entry -> {
                    result.setCriticalItemHisName(entry.getKey());
                    if (entry.getValue() != null && entry.getValue().size() > 0) {
                        entry.getValue().entrySet().forEach(subEntry -> {
                            ResultCritical resultExt = new ResultCritical();
                            BeanUtil.copyProperties(result, resultExt);
                            resultExt.setCriticalItemHisDetailName(subEntry.getKey());
                            resultMappingList.add(resultExt);
                        });
                    } else {
                        resultMappingList.add(result);
                    }
                });
            } else {
                result.setSuccess(0);
                result.setMessage("开单项缺少医院端映射；");
                resultMappingList.add(result);
            }
        }

        //模拟数据输入
        retList = createCriticalInputValue(resultMappingList);

        return retList;
    }


    public MappingShortEntity setMappingShortEntity(String name, String hisName, String hisDetailName) {
        MappingShortEntity item = new MappingShortEntity();
        item.setName(name);
        item.setHisName(hisName);
        item.setHisDetailName(hisDetailName);
        return item;
    }

    /**
     * 模拟输入数值
     * @param resultList
     * @return
     */
    public List<ResultBill> createInputValue(List<ResultBill> resultList) {
        List<ResultBill> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(resultList)) {
            return retList;
        }
        for (ResultBill result : resultList) {
            if (result.getSuccess() != null && result.getSuccess().equals(0)) {
                retList.add(result);
                continue;
            }
            if (result.getMaxValue() != null || result.getMinValue() != null) {
                if (result.getConflictItemRange() == null) {
                    result.setConflictItemRange(0);
                }
                if (result.getConflictItemRange().equals(1)) {
                    if (result.getMinValue() != null) {
                        ResultBill minResult = new ResultBill();
                        BeanUtil.copyProperties(result, minResult);
                        minResult.setInputValue(random(null, result.getMinValue()).toString());
                        BillMsg billMsg = getBillMsg(minResult);
                        minResult.setExpectedOutput(billMsg.getMsg());
                        retList.add(minResult);
                    }
                    if (result.getMaxValue() != null) {
                        ResultBill maxResult = new ResultBill();
                        BeanUtil.copyProperties(result, maxResult);
                        maxResult.setInputValue(random(result.getMaxValue(), null).toString());
                        BillMsg billMsg = getBillMsg(maxResult);
                        maxResult.setExpectedOutput(billMsg.getMsg());
                        retList.add(maxResult);
                    }
                } else {
                    result.setInputValue(random(result.getMinValue(), result.getMaxValue()).toString());
                    BillMsg billMsg = getBillMsg(result);
                    result.setExpectedOutput(billMsg.getMsg());
                    retList.add(result);
                }
            } else {
                BillMsg billMsg = getBillMsg(result);
                result.setExpectedOutput(billMsg.getMsg());
                retList.add(result);
            }
        }
        return retList;
    }

    /**
     * 模拟输入数值-危急值
     * @param resultList
     * @return
     */
    public List<ResultCritical> createCriticalInputValue(List<ResultCritical> resultList) {
        List<ResultCritical> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(resultList)) {
            return retList;
        }
        for (ResultCritical result : resultList) {
            if (result.getSuccess() != null && result.getSuccess().equals(0)) {
                retList.add(result);
                continue;
            }
            if (result.getMaxValue() != null || result.getMinValue() != null) {
                if (result.getCriticalItemRange() == null) {
                    result.setCriticalItemRange(0);
                }
                if (result.getCriticalItemRange().equals(1)) {
                    if (result.getMinValue() != null) {
                        ResultCritical minResult = new ResultCritical();
                        BeanUtil.copyProperties(result, minResult);
                        minResult.setInputValue(random(null, result.getMinValue()).toString());
                        BillMsg billMsg = getCriticalMsg(minResult);
                        minResult.setExpectedOutput(billMsg.getMsg());
                        retList.add(minResult);
                    }
                    if (result.getMaxValue() != null) {
                        ResultCritical maxResult = new ResultCritical();
                        BeanUtil.copyProperties(result, maxResult);
                        maxResult.setInputValue(random(result.getMaxValue(), null).toString());
                        BillMsg billMsg = getCriticalMsg(maxResult);
                        maxResult.setExpectedOutput(billMsg.getMsg());
                        retList.add(maxResult);
                    }
                } else {
                    result.setInputValue(random(result.getMinValue(), result.getMaxValue()).toString());
                    BillMsg billMsg = getCriticalMsg(result);
                    result.setExpectedOutput(billMsg.getMsg());
                    retList.add(result);
                }
            } else {
                BillMsg billMsg = getCriticalMsg(result);
                result.setExpectedOutput(billMsg.getMsg());
                retList.add(result);
            }
        }
        return retList;
    }

    /**
     * 模拟提示信息
     *
     * @param resultBill
     * @return
     */
    public BillMsg getBillMsg(ResultBill resultBill) {
        BillMsg billMsg = new BillMsg();
        String orderType = getOrderType(resultBill.getBillItemType());
        //输血开单
        if (resultBill.getBillItemType().equals(GraphLabelEnum.TransfusionRemind.getName())) {
            if (resultBill.getConflictItemType().equals(GraphLabelEnum.LisName.getName())) {
                String content = resultBill.getConflictItemHisName();
                if (StringUtil.isNotBlank(resultBill.getConflictItemHisDetailName())
                        && !resultBill.getConflictItemHisName().equals(resultBill.getConflictItemHisDetailName())) {
                    content += resultBill.getConflictItemHisDetailName();
                }
                if (StringUtil.isNotBlank(resultBill.getInputValue())) {
                    content += trimZero(resultBill.getInputValue());
                }
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        content,
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType
                );
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
                billMsg = MsgUtil.getCommonSurgeryMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType
                );
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoDiseaseName.getName())) {
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType
                );
            } else {
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType
                );
            }
        } else {
            if (resultBill.getConflictItemType().equals(GraphLabelEnum.BillConflictItem.getName())) {
                //互斥项
                billMsg = MsgUtil.getBillExclusionMsg(resultBill.getBillItemHisName(),
                        resultBill.getConflictItemHisName(),
                        resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType());
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.ConflictDevice.getName())) {
                //禁忌医疗器械及物品提示信息
                billMsg = MsgUtil.getBillMedEquMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
                //禁忌手术
                billMsg = MsgUtil.getCommonSurgeryMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.OralMedicine.getName())) {
                //禁忌服用药品
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        "可能正在用药" + resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.MedAllergen.getName())
                    || resultBill.getConflictItemType().equals(GraphLabelEnum.Allergen.getName())) {
                //禁忌药物过敏原/食物过敏原/过敏原
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        StringUtil.isNotBlank(resultBill.getConflictItemHisName())
                                ? resultBill.getConflictItemHisName()
                                : resultBill.getConflictItemName()
                                + "过敏",
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Group.getName())) {
                //禁忌禁忌人群
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Lis.getName())) {
                //禁忌实验室检查
                String content = "";
                if (StringUtil.isNotBlank(resultBill.getConflictItemHisName())) {
                    content = resultBill.getConflictItemHisName();
                }
                if (StringUtil.isNotBlank(resultBill.getConflictItemHisDetailName())
                        && !resultBill.getConflictItemHisName().equals(resultBill.getConflictItemHisDetailName())) {
                    content += resultBill.getConflictItemHisDetailName();
                }
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        content,
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Disease.getName())) {
                //禁忌疾病
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        resultBill.getConflictItemHisName(),
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            } else {
                //通用
                String content = resultBill.getConflictItemName();
                if (StringUtil.isNotBlank(resultBill.getInputValue())) {
                    content += trimZero(resultBill.getInputValue());
                }
                billMsg = MsgUtil.getCommonBillMsg(resultBill.getBillItemHisName(),
                        resultBill.getBillItemName(),
                        content,
                        "禁忌" + resultBill.getConflictItemType(),
                        orderType);
            }
        }
        return billMsg;
    }

    /**
     * 模拟提示信息-危急值
     * @param resultCritical
     * @return
     */
    public BillMsg getCriticalMsg(ResultCritical resultCritical) {
        BillMsg billMsg = new BillMsg();
        if (resultCritical.getCriticalItemType().equals(GraphLabelEnum.LisCritical.getName())) {
            String content = "";
            if (StringUtil.isNotBlank(resultCritical.getCriticalItemHisName())) {
                content += resultCritical.getCriticalItemHisName();
            }
            if (StringUtil.isNotBlank(resultCritical.getCriticalItemHisDetailName())
                    && !resultCritical.getCriticalItemHisName().equals(resultCritical.getCriticalItemHisDetailName())) {
                content += resultCritical.getCriticalItemHisDetailName();
            }
            if (StringUtil.isNotBlank(resultCritical.getInputValue())) {
                content += trimZero(resultCritical.getInputValue());
            }
            billMsg = MsgUtil.getCommonCriticalMsg(content, TypeEnum.lis.getName());
        } else if (resultCritical.getCriticalItemType().equals(GraphLabelEnum.PacsCritical.getName())) {
            billMsg = MsgUtil.getCommonCriticalMsg("", TypeEnum.pacs.getName());
        }
        return billMsg;
    }


    /**
     * 生成随机数
     *
     * @param min
     * @param max
     * @return
     */
    public BigDecimal random(BigDecimal min, BigDecimal max) {
        if (min == null) {
            min = BigDecimal.ZERO;
        }
        if (max == null) {
            max = min.add(BigDecimal.TEN);
        }
        BigDecimal randomVal = new BigDecimal(Math.random() * (max.doubleValue() - min.doubleValue()) + min.doubleValue());
        return randomVal.setScale(2, BigDecimal.ROUND_HALF_UP);
    }

    /**
     * 开单类型转换
     *
     * @param type
     * @return
     */
    public String getOrderType(String type) {
        String orderType = "";
        if (type.equals(GraphLabelEnum.LisSet.getName())) {
            orderType = TypeEnum.lis.getName();
        }
        if (type.equals(GraphLabelEnum.PacsName)
                || type.equals(GraphLabelEnum.PacsSubName)) {
            orderType = TypeEnum.pacs.getName();
        }
        if (type.equals(GraphLabelEnum.YiBaoDiseaseName.getName())) {
            orderType = TypeEnum.disease.getName();
        }
        if (type.equals(GraphLabelEnum.MedRegName.getName())
                || type.equals(GraphLabelEnum.Medicine.getName())) {
            orderType = TypeEnum.drug.getName();
        }
        if (type.equals(GraphLabelEnum.ClinicalFinding.getName())) {
            orderType = TypeEnum.clinical.getName();
        }
        if (type.equals(GraphLabelEnum.YiBaoOperationName.getName())) {
            orderType = TypeEnum.operation.getName();
        }
        if (type.equals(GraphLabelEnum.Vital.getName())) {
            orderType = TypeEnum.vital.getName();
        }
        if (type.equals(GraphLabelEnum.TransfusionRemind.getName())
                || type.equals(GraphLabelEnum.TransfusionSuggest.getName())) {
            orderType = TypeEnum.transfusion.getName();
        }
        if (type.equals(GraphLabelEnum.Age.getName())) {
            orderType = TypeEnum.age.getName();
        }
        return orderType;
    }

    /**
     * 去除小数点末位的0，如果都是0同时去除小数点
     *
     * @param s
     * @return
     */
    public String trimZero(String s) {
        if (s.indexOf(".") > 0) {
            //正则表达
            s = s.replaceAll("0+?$", "");//去掉后面无用的零
            s = s.replaceAll("[.]$", "");//如小数点后面全是零则去掉小数点
        }
        return s;
    }
}