package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.ResultBill;
import com.diagbot.entity.ResultInfo;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.entity.node.BillItem;
import com.diagbot.entity.node.LisCritical;
import com.diagbot.entity.node.PacsCritical;
import com.diagbot.entity.node.TransfusionRemind;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.repository.BillManRepository;
import com.diagbot.repository.LisCriticalRepository;
import com.diagbot.repository.PacsCriticalRepository;
import com.diagbot.repository.TransfusionRemindRepository;
import com.diagbot.service.ResultBillService;
import com.diagbot.service.ResultInfoService;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.RuleTestVO;
import com.google.common.collect.Lists;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/6 11:53
 */
@Component
public class RuleTestFacade {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    @Qualifier("resultBillServiceImpl")
    private ResultBillService resultBillService;
    @Autowired
    @Qualifier("resultInfoServiceImpl")
    private ResultInfoService resultInfoService;

    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    @Autowired
    private BillManRepository billManRepository;
    @Autowired
    private LisCriticalRepository lisCriticalRepository;
    @Autowired
    private PacsCriticalRepository pacsCriticalRepository;
    @Autowired
    private TransfusionRemindRepository transfusionRemindRepository;

    @Autowired
    private UnUsedMappingFacade unUsedMappingFacade;
    @Autowired
    private RuleConvertFacade ruleConvertFacade;

    @Autowired
    private SearchDataFacade searchDataFacade;


    /**
     * 全规则测试
     *
     * @param ruleTestVO
     */
    public Boolean ruleTest(RuleTestVO ruleTestVO) {
        //开单合理性规则-通用
        billRuleTest(ruleTestVO);
        //开单合理性规则-输血
        billTransfusionRuleTest(ruleTestVO);
        //危急值规则-检验
        //危急值规则-检查
        //高危药品
        //高危手术
        return true;
    }

    /**
     * 开单合理性测试
     *
     * @param ruleTestVO
     */
    public Boolean billRuleTest(RuleTestVO ruleTestVO) {
        Long hospitalId = ruleTestVO.getHospitalId();

        List<BillItem> billRules = billManRepository.getAllRules();
        //生成测试规则
        List<ResultBill> resultList = ruleConvertFacade.billRuleConvert(billRules, hospitalId);
        if (ListUtil.isEmpty(resultList)) {
            return true;
        }
        //远程调用推理
        List<ResultBill> saveList = callBill(resultList, hospitalId);
        //保存测试结果
        saveResultBill(saveList, ruleTestVO, 1);
        return true;
    }

    /**
     * 开单合理性测试-输血
     *
     * @param ruleTestVO
     */
    public Boolean billTransfusionRuleTest(RuleTestVO ruleTestVO) {
        Long hospitalId = ruleTestVO.getHospitalId();
        List<TransfusionRemind> billRules_transfusion = transfusionRemindRepository.findByStatus(StatusEnum.Enable.getKey());
        //生成测试规则
        List<ResultBill> resultList = ruleConvertFacade.billRuleConvert_transfusion(billRules_transfusion, hospitalId);
        if (ListUtil.isEmpty(resultList)) {
            return true;
        }
        //远程调用推理
        List<ResultBill> saveList = callBill(resultList, hospitalId);
        //保存测试结果
        saveResultBill(saveList, ruleTestVO, 2);
        return true;
    }

    /**
     * 危急值规则测试-检验
     *
     * @param ruleTestVO
     */
    public void criticalLisRuleTest(RuleTestVO ruleTestVO) {
        List<LisCritical> criticalRules_lis = lisCriticalRepository.findCustomByStatus(StatusEnum.Enable.getKey());
    }

    /**
     * 危急值规则测试-检查
     *
     * @param ruleTestVO
     */
    public void criticalPacsRuleTest(RuleTestVO ruleTestVO) {
        List<PacsCritical> criticalRules_pacs = pacsCriticalRepository.findByStatus(StatusEnum.Enable.getKey());
    }

    /**
     * 数据导出
     *
     * @param response
     * @param hospitalIdVO
     */
    public void exportExcel(HttpServletResponse response, HospitalIdVO hospitalIdVO) {
        Long hospitalId = hospitalIdVO.getHospitalId();
        List<LisConfig> lisConfigList = lisConfigFacade.getListByHospitalId(hospitalId);
        List<PacsConfig> pacsConfigList = pacsConfigFacade.getListByHospitalId(hospitalId);
        List<DrugConfig> drugConfigList = drugConfigFacade.getListByHospitalId(hospitalId);
        List<DiseaseConfig> diseaseConfigList = diseaseConfigFacade.getListByHospitalId(hospitalId);
        List<OperationConfig> operationConfigList = operationConfigFacade.getListByHospitalId(hospitalId);
        List<TransfusionConfig> transfusionConfigList = transfusionConfigFacade.getListByHospitalId(hospitalId);

        //开单合理性规则-通用
        List<BillItem> billRules = billManRepository.getAllRules();
        //开单合理性规则-输血
        List<TransfusionRemind> billRules_transfusion = transfusionRemindRepository.findByStatus(StatusEnum.Enable.getKey());

        //危急值规则-检验
        List<LisCritical> criticalRules_lis = lisCriticalRepository.findCustomByStatus(StatusEnum.Enable.getKey());
        //危急值规则-检查
        List<PacsCritical> criticalRules_pacs = pacsCriticalRepository.findByStatus(StatusEnum.Enable.getKey());

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("billRules", billRules);
        paramMap.put("billItems_transfusion", billRules_transfusion);
        paramMap.put("criticalRules_lis", criticalRules_lis);
        paramMap.put("criticalRules_pacs", criticalRules_pacs);

        Map<String, List<String>> outParamMap = unUsedMappingFacade.usedNames(paramMap);
        List<String> lisBillNames = outParamMap.get("lisBillNames");
        List<LisConfig> unUsedLisBill = unUsedMappingFacade.unUsedMapping(lisConfigList, lisBillNames);
        List<String> pacsBillNames = outParamMap.get("pacsBillNames");
        List<PacsConfig> unUsedPacsBill = unUsedMappingFacade.unUsedMapping(pacsConfigList, pacsBillNames);
        List<String> diseaseBillNames = outParamMap.get("diseaseBillNames");
        List<DiseaseConfig> unUsedDiseaseBill = unUsedMappingFacade.unUsedMapping(diseaseConfigList, diseaseBillNames);
        List<String> drugBillNames = outParamMap.get("drugBillNames");
        List<DrugConfig> unUsedDrugBill = unUsedMappingFacade.unUsedMapping(drugConfigList, drugBillNames);
        List<String> operationBillNames = outParamMap.get("operationBillNames");
        List<OperationConfig> unUsedOperationBill = unUsedMappingFacade.unUsedMapping(operationConfigList, operationBillNames);

        String fileName = "规则未使用的映射关系.xls";
        Map<String, Map<Class<?>, List<?>>> map = new HashMap<>();
        Map<Class<?>, List<?>> lisBillMap = new HashMap<Class<?>, List<?>>();
        lisBillMap.put(LisConfig.class, unUsedLisBill);
        map.put("检验开单项", lisBillMap);
        Map<Class<?>, List<?>> pacsBillMap = new HashMap<Class<?>, List<?>>();
        pacsBillMap.put(PacsConfig.class, unUsedPacsBill);
        map.put("检查开单项", pacsBillMap);
        Map<Class<?>, List<?>> diseaseBillMap = new HashMap<Class<?>, List<?>>();
        diseaseBillMap.put(DiseaseConfig.class, unUsedDiseaseBill);
        map.put("诊断开单项", diseaseBillMap);
        Map<Class<?>, List<?>> drugBillMap = new HashMap<Class<?>, List<?>>();
        drugBillMap.put(DrugConfig.class, unUsedDrugBill);
        map.put("药品开单项", drugBillMap);
        Map<Class<?>, List<?>> operationBillMap = new HashMap<Class<?>, List<?>>();
        operationBillMap.put(OperationConfig.class, unUsedOperationBill);
        map.put("手术和操作开单项", operationBillMap);


        ExcelUtils.exportExcel(map, null, fileName, response);
    }

    /**
     * 开单合理性远程调用接口
     *
     * @param list
     * @param hospitalId
     * @return
     */
    public List<ResultBill> callBill(List<ResultBill> list, Long hospitalId) {
        List<ResultBill> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(list)) {
            return retList;
        }
        for (ResultBill resultBill : list) {
            if (resultBill.getSuccess() != null
                    && resultBill.getSuccess().equals(0)) {
                retList.add(resultBill);
                continue;
            }
            IndicationPushVO indicationPushVO = searchDataFacade.creatBillVO(resultBill);
            indicationPushVO.setHospitalId(hospitalId);

            resultBill.setInputParams(FastJsonUtils.getBeanToJson(indicationPushVO));

            RespDTO<IndicationDTO> respDTO = cdssCoreClient.indication(indicationPushVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                IndicationDTO data = respDTO.data;
                List<BillMsg> billMsgList = data.getBillMsgList();
                if (ListUtil.isEmpty(billMsgList)) {
                    resultBill.setSuccess(0);
                    resultBill.setMessage("未提示开单项");
                    retList.add(resultBill);
                } else {
                    for (BillMsg billMsg : billMsgList) {
                        ResultBill saveResultBill = new ResultBill();
                        BeanUtil.copyProperties(resultBill, saveResultBill);
                        saveResultBill.setOutput(billMsg.getMsg());
                        if (saveResultBill.getOutput().equals(saveResultBill.getExpectedOutput())) {
                            saveResultBill.setSuccess(1);
                        } else {
                            saveResultBill.setSuccess(0);
                            saveResultBill.setMessage("提示内容与预期不符");
                        }
                        retList.add(saveResultBill);
                    }
                }
            } else {
                resultBill.setSuccess(0);
                resultBill.setMessage("远程调用推理接口失败");
                retList.add(resultBill);
            }
        }
        return retList;
    }

    /**
     * 保存开单合理性测试结果
     * @param list
     * @param ruleTestVO
     * @param billType
     * @return
     */
    public Boolean saveResultBill(List<ResultBill> list, RuleTestVO ruleTestVO, Integer billType) {
        //保存测试结果
        Date now = new Date();
        Integer totleNum = list.size();
        Integer successNum = list.stream().filter(i -> i.getSuccess().equals(1)).collect(Collectors.toList()).size();
        Integer failedNum = list.stream().filter(i -> i.getSuccess().equals(0)).collect(Collectors.toList()).size();
        QueryWrapper<ResultInfo> resultInfoQueryWrapper = new QueryWrapper<>();
        resultInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", ruleTestVO.getHospitalId())
                .eq("case_id", ruleTestVO.getCaseId());
        ResultInfo resultInfo = resultInfoService.getOne(resultInfoQueryWrapper, false);
        if (resultInfo == null) {
            resultInfo = new ResultInfo();
            resultInfo.setHospitalId(ruleTestVO.getHospitalId());
            resultInfo.setCaseId(ruleTestVO.getCaseId());
            resultInfo.setGmtCreate(now);
        }
        resultInfo.setGmtModified(now);
        resultInfo.setRuleNum(totleNum);
        resultInfo.setRuleSuccessNum(successNum);
        resultInfo.setRuleFailedNum(failedNum);
        resultInfoService.saveOrUpdate(resultInfo);

        QueryWrapper<ResultBill> resultBillQueryWrapper = new QueryWrapper<>();
        resultBillQueryWrapper.eq("result_id", resultInfo.getId());
        resultBillService.remove(resultBillQueryWrapper);

        for (ResultBill resultBill : list) {
            resultBill.setBillType(billType);
            resultBill.setGmtCreate(now);
            resultBill.setGmtModified(now);
            resultBill.setResultId(resultInfo.getId());
        }

        resultBillService.saveOrUpdateBatch(list);
        return true;
    }
}