package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.entity.ResultBill;
import com.diagbot.enums.GraphLabelEnum;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.Drug;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.ItemExt;
import org.springframework.stereotype.Component;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/11 11:37
 */
@Component
public class SearchDataFacade {

    /**
     * 生成开单合理性入参
     *
     * @param resultBill
     * @return
     */
    public IndicationPushVO creatBillVO(ResultBill resultBill) {
        IndicationPushVO indicationPushVO = new IndicationPushVO();
        indicationPushVO.setAge("20");
        indicationPushVO.setSex(3);
        indicationPushVO.setRuleType("2");

        //开单项
        if (resultBill.getBillItemType().equals(GraphLabelEnum.LisSet.getName())) {
            //检验开单项
            Lis lis = new Lis();
            lis.setName(resultBill.getBillItemHisName());
            lis.setDetailName(resultBill.getBillItemHisDetailName());
            lis.setUniqueName(resultBill.getBillItemName());
            indicationPushVO.getLisOrder().add(lis);
        }
        if (resultBill.getBillItemType().equals(GraphLabelEnum.PacsName.getName())
                || resultBill.getBillItemType().equals(GraphLabelEnum.PacsSubName.getName())) {
            Pacs pacs = new Pacs();
            pacs.setName(resultBill.getBillItemHisName());
            pacs.setUniqueName(resultBill.getBillItemName());
            indicationPushVO.getPacsOrder().add(pacs);
        }
        if (resultBill.getBillItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
            Item operation = new Item();
            operation.setName(resultBill.getBillItemHisName());
            operation.setUniqueName(resultBill.getBillItemName());
            indicationPushVO.getOperationOrder().add(operation);
        }
        if (resultBill.getBillItemType().equals(GraphLabelEnum.MedRegName.getName())) {
            Drug drug = new Drug();
            drug.setName(resultBill.getBillItemHisName());
            drug.setUniqueName(resultBill.getBillItemName());
            indicationPushVO.getDrugOrder().add(drug);
        }
        if (resultBill.getBillItemType().equals(GraphLabelEnum.TransfusionRemind.getName())) {
            ItemExt itemExt = new ItemExt();
            itemExt.setName(resultBill.getBillItemHisName());
            itemExt.setUniqueName(resultBill.getBillItemName());
            indicationPushVO.getTransfusionOrder().add(itemExt);
        }

        //未使用禁忌条件：食物过敏原、手术、给药途径
        if (resultBill.getConflictItemType().equals(GraphLabelEnum.Gender.getName())) {
            if (resultBill.getConflictItemName().equals("男")) {
                indicationPushVO.setSex(1);
            } else if (resultBill.getConflictItemName().equals("女")) {
                indicationPushVO.setSex(2);
            } else {
                indicationPushVO.setSex(3);
            }
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Vital.getName())) {
            //禁忌年龄特殊处理
            if (resultBill.getConflictItemName().equals("年龄")) {
                indicationPushVO.setAge(resultBill.getInputValue());
            } else if (resultBill.getConflictItemName().equals("体温")) {
                indicationPushVO.setVital(resultBill.getConflictItemName() + resultBill.getInputValue() + "℃,");
            } else {
                indicationPushVO.setVital(resultBill.getConflictItemName() + resultBill.getInputValue() + ",");
            }
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.ClinicalFinding.getName())) {
            indicationPushVO.setSymptom(indicationPushVO.getSymptom() +
                    resultBill.getConflictItemName() +
                    (resultBill.getInputValue() != null ? resultBill.getInputValue() : "") +
                    ",");
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.OralMedicine.getName())) {
            indicationPushVO.setSymptom(indicationPushVO.getSymptom() +
                    "一直服用" +
                    (StringUtil.isNotBlank(resultBill.getConflictItemHisName())
                            ? resultBill.getConflictItemHisName()
                            : resultBill.getConflictItemName()) +
                    ",");
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.MedAllergen.getName())
                || resultBill.getConflictItemType().equals(GraphLabelEnum.Allergen.getName())) {
            indicationPushVO.setPasts(indicationPushVO.getPasts() +
                    "对" +
                    (StringUtil.isNotBlank(resultBill.getConflictItemHisName())
                            ? resultBill.getConflictItemHisName()
                            : resultBill.getConflictItemName()) +
                    "过敏,");
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.PacsDescribe.getName())) {
            Pacs pacs = new Pacs();
            pacs.setName("胸部CT");
            pacs.setUniqueName("胸部CT");
            pacs.setDesc(resultBill.getConflictItemName());
            pacs.setResult(resultBill.getConflictItemName());
            indicationPushVO.getPacs().add(pacs);
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Lis.getName())
                || resultBill.getConflictItemType().equals(GraphLabelEnum.LisName.getName())) {
            Lis lis = new Lis();
            lis.setName(resultBill.getConflictItemHisName());
            lis.setDetailName(resultBill.getConflictItemHisDetailName());
            lis.setUniqueName(resultBill.getConflictItemName());
            lis.setUnits(resultBill.getUnit());
            if (resultBill.getMinValue() != null || resultBill.getMaxValue() != null) {
                lis.setValue(Double.valueOf(resultBill.getInputValue()));
            } else {
                lis.setOtherValue(resultBill.getInputValue());
            }
            if (resultBill.getMaxValue() != null) {
                lis.setMaxValue(resultBill.getMaxValue().doubleValue());
            }
            if (resultBill.getMinValue() != null) {
                lis.setMinValue(resultBill.getMinValue().doubleValue());
            }
            indicationPushVO.getLis().add(lis);
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Disease.getName())
                || resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoDiseaseName.getName())) {
            Item diag = new Item();
            diag.setName(resultBill.getConflictItemHisName());
            diag.setUniqueName(resultBill.getConflictItemName());
            indicationPushVO.getDiag().add(diag);
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.Group.getName())) {
            indicationPushVO.setSymptom(indicationPushVO.getSymptom() +
                    resultBill.getConflictItemName() +
                    (resultBill.getInputValue() != null ? resultBill.getInputValue() : "") +
                    (StringUtil.isNotBlank(resultBill.getUnit()) ? resultBill.getUnit() : "") +
                    ",");
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.ConflictDevice.getName())) {
            indicationPushVO.setPasts(indicationPushVO.getPasts() +
                    "有" + resultBill.getConflictItemName() + ",");
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.BillConflictItem.getName())) {
            Pacs pacs = new Pacs();
            pacs.setName(resultBill.getConflictItemHisName());
            pacs.setUniqueName(resultBill.getConflictItemName());
            indicationPushVO.getPacsOrder().add(pacs);
        } else if (resultBill.getConflictItemType().equals(GraphLabelEnum.YiBaoOperationName.getName())) {
            Item operation = new Item();
            operation.setName(resultBill.getConflictItemHisName());
            operation.setUniqueName(resultBill.getConflictItemName());
            indicationPushVO.getOperation().add(operation);
        }
        return indicationPushVO;
    }
}