package com.diagbot.facade;

import com.diagbot.entity.node.BillItem;
import com.diagbot.entity.node.MedNameRegName;
import com.diagbot.enums.GraphLabelEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/7 15:32
 */
@Component
public class UnUsedMappingFacade {

    @Autowired
    private MedicineRepository medicineRepository;

    /**
     * 未使用映射关系
     *
     * @param items
     * @param usedNames
     * @param <T>
     * @return
     */
    public <T> List<T> unUsedMapping(List<T> items, List<String> usedNames) {
        List<T> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(usedNames)) {
            return items;
        }
        Map<String, List<T>> map = EntityUtil.makeEntityListMap(items, "uniqueName");
        for (Map.Entry<String, List<T>> entry : map.entrySet()) {
            if (!usedNames.contains(entry.getKey())) {
                retList.addAll(entry.getValue());
            }
        }
        return retList;
    }

    /**
     * 规则中包含的标准术语
     *
     * @param map
     * @return
     */
    public Map<String, List<String>> usedNames(Map<String, Object> map) {
        Map<String, List<String>> retMap = new HashMap<>();
        if (map.containsKey("billRules")) {
            List<BillItem> billItems = (List<BillItem>) map.get("billRules");
            //开单项使用
            List<String> lisBillNames = billItems.stream()
                    .filter(i -> i.getConceptLabel().equals(GraphLabelEnum.LisSet.getName()))
                    .map(i -> i.getConceptName())
                    .distinct()
                    .collect(Collectors.toList());
            //禁忌项使用
            lisBillNames.addAll(billItems.stream()
                    .filter(i -> i.getConditionLabel().equals(GraphLabelEnum.Lis.getName()))
                    .map(i -> i.getConditionName())
                    .distinct()
                    .collect(Collectors.toList()));
            //开单项使用
            List<String> pacsBillNames = billItems.stream()
                    .filter(i -> i.getConceptLabel().equals(GraphLabelEnum.PacsName.getName()) || i.getConceptLabel().equals(GraphLabelEnum.PacsSubName.getName()))
                    .map(i -> i.getConceptName())
                    .distinct()
                    .collect(Collectors.toList());
            //禁忌项使用
            pacsBillNames.addAll(billItems.stream()
                    .filter(i -> i.getConditionLabel().equals(GraphLabelEnum.BillConflictItem.getName()))
                    .map(i -> i.getConditionName())
                    .distinct()
                    .collect(Collectors.toList()));
            //开单项使用
            List<String> operationBillNames = billItems.stream()
                    .filter(i -> i.getConceptLabel().equals(GraphLabelEnum.YiBaoOperationName.getName()))
                    .map(i -> i.getConceptName())
                    .distinct()
                    .collect(Collectors.toList());
            //禁忌项使用
            /*operationBillNames.addAll(billItems.stream()
                    .filter(i -> i.getConditionLabel().equals("手术"))
                    .map(i -> i.getConditionName())
                    .distinct()
                    .collect(Collectors.toList()));*/
            //开单项使用
            List<String> diseaseBillNames = billItems.stream()
                    .filter(i -> i.getConceptLabel().equals(GraphLabelEnum.YiBaoDiseaseName.getName()))
                    .map(i -> i.getConceptName())
                    .distinct()
                    .collect(Collectors.toList());
            //禁忌项使用
            diseaseBillNames.addAll(billItems.stream()
                    .filter(i -> i.getConditionLabel().equals(GraphLabelEnum.Disease.getName()))
                    .map(i -> i.getConditionName())
                    .distinct()
                    .collect(Collectors.toList()));
            //开单项使用
            List<String> drugRegNames = billItems.stream()
                    .filter(i -> i.getConceptLabel().equals(GraphLabelEnum.MedRegName.getName()))
                    .map(i -> i.getConceptName())
                    .distinct()
                    .collect(Collectors.toList());
            //药品注册名转药品通用名
            List<String> drugBillNames = Lists.newArrayList();
            if (ListUtil.isNotEmpty(drugBillNames)) {
                List<MedNameRegName> medNameRegNames = medicineRepository.findCustomByNameAndRegName(StatusEnum.Enable.getKey());
                Map<String, List<MedNameRegName>> medNameRegNameMap
                        = EntityUtil.makeEntityListMap(medNameRegNames, "medRegName");
                for (String regName : drugRegNames) {
                    if (medNameRegNameMap.containsKey(regName)) {
                        drugBillNames.addAll(medNameRegNameMap.get(regName).stream().map(i -> i.getMedName()).collect(Collectors.toList()));
                    }
                }
            }
            drugBillNames.addAll(billItems.stream()
                    .filter(i -> i.getConditionLabel().equals(GraphLabelEnum.OralMedicine.getName()) || i.getConditionLabel().equals(GraphLabelEnum.MedAllergen.getName()))
                    .map(i -> i.getConditionName())
                    .distinct()
                    .collect(Collectors.toList()));
            retMap.put("lisBillNames", lisBillNames);
            retMap.put("pacsBillNames", pacsBillNames);
            retMap.put("diseaseBillNames", diseaseBillNames);
            retMap.put("drugBillItems", drugBillNames);
            retMap.put("operationBillNames", operationBillNames);
        }
        return retMap;
    }
}
