package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.CaseResultDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ResultBill;
import com.diagbot.facade.ResultBillFacade;
import com.diagbot.facade.ResultInfoFacade;
import com.diagbot.facade.RuleTestFacade;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.ResultBillPageVO;
import com.diagbot.vo.RuleTestVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/7 19:46
 */
@RestController
@RequestMapping("/test/rule")
@Api(value = "规则测试相关API", tags = { "规则测试相关API" })
public class RuleTestController {
    @Autowired
    private RuleTestFacade ruleTestFacade;
    @Autowired
    private ResultInfoFacade resultInfoFacade;
    @Autowired
    private ResultBillFacade resultBillFacade;

    @ApiOperation(value = "开单合理性测试列表[zhaops]",
            notes = "success： 成功标志（0：失败，1：成功） <br>" +
                    "resultId： 结果id <br>" +
                    "billType： 开单项类型（1：通用，2：输血） <br>")
    @PostMapping("/getResultBillPage")
    @SysLogger("getResultBillPage")
    public RespDTO<IPage<ResultBill>> getResultBillPage(@Valid @RequestBody ResultBillPageVO resultBillPageVO) {
        IPage<ResultBill> data = resultBillFacade.getPage(resultBillPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医院规则测试结果列表[zhaops]",
            notes = "hospitalId： 医院Id <br>")
    @PostMapping("/getCaseResultList")
    @SysLogger("getCaseResultList")
    public RespDTO<List<CaseResultDTO>> getCaseResultList(@Valid @RequestBody RuleTestVO ruleTestVO) {
        List<CaseResultDTO> data = resultInfoFacade.getCaseResultList(ruleTestVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 数据导出
     *
     * @param response
     * @param hospitalIdVO
     */
    @ApiOperation(value = "导出规则未使用映射关系[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response, @RequestBody @Valid HospitalIdVO hospitalIdVO) {
        ruleTestFacade.exportExcel(response, hospitalIdVO);
    }

    @ApiOperation(value = "全规则测试[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    "caseId： 测试用例id<br>")
    @PostMapping("/ruleTest")
    @SysLogger("ruleTest")
    public RespDTO<Boolean> ruleTest(@Valid @RequestBody RuleTestVO ruleTestVO) {
        Boolean data = ruleTestFacade.ruleTest(ruleTestVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "开单合理性规则测试[zhaops]",
            notes = "hospitalId： 医院Id <br>" +
                    "caseId： 测试用例id<br>" +
                    "billType： 开单项测试规则类型（1：通用，2：输血） <br>")
    @PostMapping("/billRuleTest")
    @SysLogger("billRuleTest")
    public RespDTO<Boolean> billRuleTest(@Valid @RequestBody RuleTestVO ruleTestVO) {
        Boolean data = false;
        if (ruleTestVO.getBillType().equals(1)) {
            data = ruleTestFacade.billRuleTest(ruleTestVO);
        } else if (ruleTestVO.getBillType().equals(2)) {
            data = ruleTestFacade.billTransfusionRuleTest(ruleTestVO);
        }
        return RespDTO.onSuc(data);
    }
}